import produce from "immer";
import React, { PureComponent } from "react";
import { Subject } from "rxjs";
import { startWith, takeUntil, map, tap } from "rxjs/operators";
export class InputContainer extends PureComponent {
  constructor(props) {
    super(props);
    this.destroy$ = new Subject();
    this.state = {
      status: 'PROCESSED'
    };
    this.initializeControl.bind(this);
  }

  componentDidMount() {
    const control = this.props.control;
    control && this.initializeControl(control);
  }

  initializeControl(control) {
    control.statusChanges.pipe(takeUntil(this.destroy$), startWith(control.status), map(statusType => {
      if (statusType === 'INVALID') {
        // INVALID, PENDING, DISABLED,  VALID
        return {
          status: 'ERROR',
          error: control.errors
        };
      } else if (statusType === 'PENDING') {
        return {
          status: 'PROCESSING',
          data: {}
        };
      }

      return {
        status: 'PROCESSED',
        data: {}
      };
    }), tap(status => {
      this.setState(produce(draft => {
        draft.data = status.data;
        draft.error = status.error;
        draft.status = status.status;
      }));
    })).subscribe();
  }

  componentDidUpdate(prevProps, prevState) {
    let newControl = this.props.control;

    if (newControl && prevProps.control !== newControl) {
      this.initializeControl(newControl);
    }
  }

  componentWillUnmount() {
    this.destroy$.next();
    this.destroy$.complete();
  }

  render() {
    const form = this.props.formState ? this.props.formState : {
      submitted: true,
      loadStatus: {
        status: 'PROCESSED'
      }
    };
    const dataProcessingStatus = this.props.dataProcessingStatus || {
      status: 'PROCESSED'
    };
    const renderInput = this.props.renderInput;
    const control = this.props.control;
    const data = dataProcessingStatus.data;
    const processingTemplate = this.props.processingTemplate;
    const showErrorsFn = this.props.showErrorsFn ? this.props.showErrorsFn : params => {
      return true;
    };
    const showErrors = showErrorsFn({
      form: {
        submitted: this.props.formState.submitted
      },
      input: {
        dirty: (control == null ? void 0 : control.dirty) || false,
        touched: (control == null ? void 0 : control.touched) || false
      }
    });
    const errorsTemplate = this.props.errorsTemplate;
    const labelTemplate = this.props.labelTemplate;
    const reloadTemplate = this.props.reloadTemplate;
    return /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement("div", {
      style: this.props.style || {},
      className: this.props.className || ''
    }, form.loadStatus.status !== 'ERROR' && /*#__PURE__*/React.createElement(React.Fragment, null, labelTemplate && labelTemplate({
      showErrors,
      validationStatus: this.state.status,
      label: this.props.label
    }), form.loadStatus.status === 'PROCESSED' && dataProcessingStatus.status === 'PROCESSED' && renderInput({
      control,
      data,
      validationStatus: this.state.status,
      showErrors
    }), (form.loadStatus.status === 'PROCESSING' || dataProcessingStatus.status === 'PROCESSING') && processingTemplate && processingTemplate(), form.loadStatus.status === 'PROCESSED' && dataProcessingStatus.status === 'PROCESSED' && showErrors && errorsTemplate && errorsTemplate({
      errors: this.state.error,
      form: {
        submitted: !!form.submitted
      },
      input: {
        touched: control.touched,
        dirty: control.dirty
      }
    }), dataProcessingStatus.status === 'ERROR' && reloadTemplate && reloadTemplate({
      reloadFn: () => {}
    }))));
  }

}