import _extends from "@babel/runtime/helpers/esm/extends";
import produce from "immer";
import { catchError, filter, forkJoin, from, map, mapTo, merge, of, Subject, switchMap, take, takeUntil, tap, zip } from "rxjs";
import { ModuleUtils } from "../../utils/lang";
export class EntitySearchClass {
  constructor(props, setState) {
    this.props = props;
    this.setState = setState;
    this.destroy$ = new Subject();
    this.reloadResource$ = new Subject();
    this.newState = {
      data: [],
      loadStatus: {},
      paging: {
        page: 0,
        size: 1000,
        sort: null
      },
      params: {},
      resources: {}
    };
    this.reloadResource = this.reloadResource.bind(this);
  }

  reloadResource(resourceId) {}

  init() {
    const resources = this.props.resources || {};
    Object.keys(resources).map(key => _extends({}, resources[key], {
      key
    })).filter(resource => !resource.concurrent).forEach(resource => {
      this.setState(prevState => {
        const newState = produce(draft => {
          draft.resources[resource.key] = {};
        })(prevState);
        this.newState = newState;
        return newState;
      });
      const processingStatus$ = new Subject();
      merge(processingStatus$.pipe(mapTo({
        status: 'PROCESSING'
      })), merge(this.reloadResource$.pipe(filter(reload => reload.id === resource.key), map(reload => reload.params)), resource.params$).pipe(takeUntil(this.destroy$), tap(params => {
        // this.setState(
        //     (prevState) => {
        //         const newState = produce(
        //             (draft: IEntitySearchState) => {
        //                 draft.resourceParams[resource.key] = params;
        //             }
        //         )(prevState);
        //         this.newState = newState;
        //         return newState
        //     }
        // )
        processingStatus$.next();
      }), switchMap(params => {
        return resource.loadData$(params).pipe( // take(1),
        map(data => {
          alert(2);
          return {
            status: 'PROCESSED',
            data
          };
        }), catchError(error => {
          alert(9);
          return of({
            status: 'ERROR',
            error
          });
        }));
      }))).pipe(takeUntil(this.destroy$), tap(processingStatus => {
        alert(1);
        resource.onProcessingStatus && resource.onProcessingStatus(processingStatus, {});
        this.setState(prevState => {
          const newState = produce(draft => {
            draft.resources[resource.key].status = processingStatus.status;
            draft.resources[resource.key].data = processingStatus.data;
            draft.resources[resource.key].error = processingStatus.error;
          })(prevState);
          this.newState = newState;
          return newState;
        });
      })).subscribe();
    });
    /******************************************************************************************************* */

    /******************************************************************************************************* */

    /******************************************************************************************************* */

    const defaultAdaptLoadData = (data, resources) => ({
      data,
      resources: {}
    });

    const adaptLoadData = this.props.adaptLoadData ? this.props.adaptLoadData : defaultAdaptLoadData;
    const processingStatus$ = new Subject();
    merge(processingStatus$.pipe(mapTo({
      status: 'PROCESSING'
    })), this.props.params$.pipe(takeUntil(this.destroy$), tap(params => {
      this.setState(prevState => {
        const newState = produce(draft => {
          draft.params = params;
        })(prevState);
        this.newState = newState;
        return newState;
      });
      processingStatus$.next();
    }), switchMap(params => {
      /********************************* */
      const resources = this.props.resources || {};
      const concurrentResources = Object.keys(resources).map(key => _extends({}, resources[key], {
        key
      })).filter(resource => resource.concurrent);
      const loadResources$ = concurrentResources.map(resource => {
        if (resource.cache) {
          const resourceOld = this.newState.resources[resource.key];

          if (resourceOld && resourceOld.status === 'PROCESSED') {
            return of({
              status: 'PROCESSED',
              data: {
                key: resource.key,
                data: resourceOld.data
              }
            });
          }
        }

        return resource.loadData$(params).pipe(map(data => {
          return {
            status: 'PROCESSED',
            data: {
              key: resource.key,
              data
            }
          };
        }), catchError(error => {
          return of({
            status: 'PROCESSED',
            error: {
              key: resource.key,
              error
            }
          });
        }));
      });
      /********************************* */

      const concurrentResources$ = (loadResources$.length ? forkJoin(loadResources$) : of([])).pipe(map(resources => {
        return resources.reduce((t, e) => {
          var _e$data, _e$error, _e$data2, _e$error2;

          t[((_e$data = e.data) == null ? void 0 : _e$data.key) || ((_e$error = e.error) == null ? void 0 : _e$error.key)] = {
            status: e.status,
            data: (_e$data2 = e.data) == null ? void 0 : _e$data2.data,
            error: (_e$error2 = e.error) == null ? void 0 : _e$error2.error
          };
          return t;
        }, {});
      }));
      const data$ = this.props.loadData$(params);
      const loadData$ = ModuleUtils.isObservable(data$) ? data$ : ModuleUtils.isPromise(data$) ? from(data$) : of(data$);
      return zip(loadData$, concurrentResources$).pipe(take(1), map(([data, resources]) => adaptLoadData(data, resources)), map(data => {
        return {
          status: 'PROCESSED',
          data
        };
      }), catchError(error => {
        return of({
          status: 'ERROR',
          error
        });
      }));
    }))).pipe(takeUntil(this.destroy$), tap(processingStatus => {
      this.props.onProcessingStatus && this.props.onProcessingStatus(processingStatus, {});

      if (processingStatus.status === 'PROCESSED') {
        this.setState(prevState => {
          const newState = produce(draft => {
            const resources = processingStatus.data.resources;
            Object.keys(resources || {}).map(key => ({
              key,
              data: resources[key]
            })).forEach(item => {
              draft.resources[item.key] = {
                status: 'PROCESSED',
                data: item.data
              };
            });
          })(prevState);
          this.newState = newState;
          return newState;
        });
      }

      this.setState(prevState => {
        const newState = produce(draft => {
          var _processingStatus$dat, _processingStatus$dat2;

          draft.loadStatus.status = processingStatus.status;
          draft.loadStatus.data = (_processingStatus$dat = processingStatus.data) == null ? void 0 : _processingStatus$dat.data;
          draft.loadStatus.error = processingStatus.error;
          draft.data = (_processingStatus$dat2 = processingStatus.data) == null ? void 0 : _processingStatus$dat2.data;
        })(prevState);
        this.newState = newState;
        return newState;
      });
    })).subscribe();
  }

  close() {
    this.destroy$.next();
    this.destroy$.complete();
  }

  get config() {
    return {
      searchStatus: this.newState
    };
  }

}