"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

exports.__esModule = true;
exports.InputContainer = void 0;

var _immer = _interopRequireDefault(require("immer"));

var _react = _interopRequireWildcard(require("react"));

var _rxjs = require("rxjs");

var _operators = require("rxjs/operators");

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

class InputContainer extends _react.PureComponent {
  constructor(props) {
    super(props);
    this.destroy$ = new _rxjs.Subject();
    this.state = {
      status: 'PROCESSED'
    };
    this.initializeControl.bind(this);
  }

  componentDidMount() {
    const control = this.props.control;
    control && this.initializeControl(control);
  }

  initializeControl(control) {
    control.statusChanges.pipe((0, _operators.takeUntil)(this.destroy$), (0, _operators.startWith)(control.status), (0, _operators.map)(statusType => {
      if (statusType === 'INVALID') {
        // INVALID, PENDING, DISABLED,  VALID
        return {
          status: 'ERROR',
          error: control.errors
        };
      } else if (statusType === 'PENDING') {
        return {
          status: 'PROCESSING',
          data: {}
        };
      }

      return {
        status: 'PROCESSED',
        data: {}
      };
    }), (0, _operators.tap)(status => {
      this.setState((0, _immer.default)(draft => {
        draft.data = status.data;
        draft.error = status.error;
        draft.status = status.status;
      }));
    })).subscribe();
  }

  componentDidUpdate(prevProps, prevState) {
    let newControl = this.props.control;

    if (newControl && prevProps.control !== newControl) {
      this.initializeControl(newControl);
    }
  }

  componentWillUnmount() {
    this.destroy$.next();
    this.destroy$.complete();
  }

  render() {
    const form = this.props.formState ? this.props.formState : {
      submitted: true,
      loadStatus: {
        status: 'PROCESSED'
      }
    };
    const dataProcessingStatus = this.props.dataProcessingStatus || {
      status: 'PROCESSED'
    };
    const renderInput = this.props.renderInput;
    const control = this.props.control;
    const data = dataProcessingStatus.data;
    const processingTemplate = this.props.processingTemplate;
    const showErrorsFn = this.props.showErrorsFn ? this.props.showErrorsFn : params => {
      return true;
    };
    const showErrors = showErrorsFn({
      form: {
        submitted: this.props.formState.submitted
      },
      input: {
        dirty: (control == null ? void 0 : control.dirty) || false,
        touched: (control == null ? void 0 : control.touched) || false
      }
    });
    const errorsTemplate = this.props.errorsTemplate;
    const labelTemplate = this.props.labelTemplate;
    const reloadTemplate = this.props.reloadTemplate;
    return /*#__PURE__*/_react.default.createElement(_react.default.Fragment, null, /*#__PURE__*/_react.default.createElement("div", {
      style: this.props.style || {},
      className: this.props.className || ''
    }, form.loadStatus.status !== 'ERROR' && /*#__PURE__*/_react.default.createElement(_react.default.Fragment, null, labelTemplate && labelTemplate({
      showErrors,
      validationStatus: this.state.status,
      label: this.props.label
    }), form.loadStatus.status === 'PROCESSED' && dataProcessingStatus.status === 'PROCESSED' && renderInput({
      control,
      data,
      validationStatus: this.state.status,
      showErrors
    }), (form.loadStatus.status === 'PROCESSING' || dataProcessingStatus.status === 'PROCESSING') && processingTemplate && processingTemplate(), form.loadStatus.status === 'PROCESSED' && dataProcessingStatus.status === 'PROCESSED' && showErrors && errorsTemplate && errorsTemplate({
      errors: this.state.error,
      form: {
        submitted: !!form.submitted
      },
      input: {
        touched: control.touched,
        dirty: control.dirty
      }
    }), dataProcessingStatus.status === 'ERROR' && reloadTemplate && reloadTemplate({
      reloadFn: () => {}
    }))));
  }

}

exports.InputContainer = InputContainer;