"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

exports.__esModule = true;
exports.EntityFormClass = void 0;

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _immer = _interopRequireDefault(require("immer"));

var _rxjs = require("rxjs");

var _lang = require("../../utils/lang");

const retryProcessing = config => {
  const retryPipeline = (0, _rxjs.retryWhen)(errors => errors.pipe( // Use concat map to keep the errors in order and make sure they
  // aren't executed in parallel
  (0, _rxjs.concatMap)((e, i) => // Executes a conditional Observable depending on the result
  // of the first argument
  (0, _rxjs.iif)(() => i > 10, // If the condition is true we throw the error (the last error)
  (0, _rxjs.throwError)(e), // Otherwise we pipe this back into our stream and delay the retry
  (0, _rxjs.of)(e).pipe((0, _rxjs.delay)(500))))));
  return retryPipeline;
};

class EntityFormClass {
  constructor(props, setState) {
    this.props = props;
    this.setState = setState;
    this.loadResource$ = new _rxjs.Subject();
    this.destroy$ = new _rxjs.Subject();
    this.reloadResource$ = new _rxjs.Subject();
    this.triggerSave$ = new _rxjs.Subject();
    this.triggerLoad$ = new _rxjs.Subject();
    this.newState = {
      loadStatus: {},
      params: {},
      resourceParams: {},
      resources: {},
      saveStatus: {},
      submitted: false
    };
    this.reloadResource = this.reloadResource.bind(this);
    this.configSave = this.configSave.bind(this);
    this.loadResourceConfig = this.loadResourceConfig.bind(this);
  }

  loadResourceConfig() {
    const processingStatus$ = new _rxjs.Subject();
    (0, _rxjs.merge)(processingStatus$.pipe((0, _rxjs.map)(config => ({
      status: 'PROCESSING',
      config
    }))), this.loadResource$.pipe((0, _rxjs.tap)(config => {
      processingStatus$.next(config);
    }), (0, _rxjs.switchMap)(config => {
      const data$ = config.config.data$ ? config.config.data$(config.params) : (0, _rxjs.of)(config.params);
      const loadData$ = _lang.ModuleUtils.isObservable(data$) ? data$ : _lang.ModuleUtils.isPromise(data$) ? (0, _rxjs.from)(data$) : (0, _rxjs.of)(data$);
      const retryConfig = config.config.retry ? config.config.retry : {
        interval: 100
      };
      return loadData$.pipe(retryProcessing(retryConfig), (0, _rxjs.take)(1), (0, _rxjs.map)(data => {
        return {
          status: 'PROCESSED',
          data,
          config
        };
      }), (0, _rxjs.catchError)(error => {
        return (0, _rxjs.of)({
          status: 'ERROR',
          error,
          config
        });
      }));
    }))).pipe((0, _rxjs.takeUntil)(this.destroy$), (0, _rxjs.tap)(processingStatus => {
      processingStatus.config.config.onProcessingStatus(processingStatus); // processingStatus.data&&processingStatus.data.config&&processingStatus.data.config.config&&processingStatus.data.config.config.onProcessingStatus(processingStatus)

      this.setState(prevState => {
        const newState = (0, _immer.default)(draft => {
          const config = processingStatus.config;
          draft.resources[config.id] = {
            status: processingStatus.status,
            data: processingStatus.data,
            error: processingStatus.error
          };
        })(prevState);
        this.newState = newState;
        return newState;
      });
    })).subscribe();
  }

  init() {
    this.loadResourceConfig();
    this.configSave();
    const loadConfig = this.props.load;

    if (!loadConfig) {
      this.setState(prevState => {
        const newState = (0, _immer.default)(draft => {
          draft.loadStatus.status = 'PROCESSED';
          draft.loadStatus.data = {};
        })(prevState);
        this.newState = newState;
        return newState;
      });
      return;
    }

    const resources = loadConfig.resources || {};
    setTimeout(() => Object.keys(resources).map(key => (0, _extends2.default)({}, resources[key], {
      key
    })).forEach(resource => {
      this.setState(prevState => {
        const newState = (0, _immer.default)(draft => {
          draft.resources[resource.key] = {};
        })(prevState);
        this.newState = newState;
        return newState;
      });
      const processingStatus$ = new _rxjs.Subject();
      (0, _rxjs.merge)(processingStatus$.pipe((0, _rxjs.mapTo)({
        status: 'PROCESSING'
      })), (0, _rxjs.merge)(this.reloadResource$.pipe((0, _rxjs.filter)(reload => reload.id === resource.key), (0, _rxjs.map)(reload => reload.params)), resource.params$) // resource.params$
      .pipe((0, _rxjs.takeUntil)(this.destroy$), (0, _rxjs.tap)(params => {
        this.setState(prevState => {
          const newState = (0, _immer.default)(draft => {
            draft.resourceParams[resource.key] = params;
          })(prevState);
          this.newState = newState;
          return newState;
        });
        processingStatus$.next();
      }), (0, _rxjs.switchMap)(params => {
        const data$ = resource.data$ ? resource.data$(params) : (0, _rxjs.of)(params);
        const loadData$ = _lang.ModuleUtils.isObservable(data$) ? data$ : _lang.ModuleUtils.isPromise(data$) ? (0, _rxjs.from)(data$) : (0, _rxjs.of)(data$);
        const retryConfig = resource.retry ? resource.retry : {
          interval: 1000
        };
        return loadData$.pipe(retryProcessing(retryConfig), (0, _rxjs.take)(1), (0, _rxjs.map)(data => {
          return {
            status: 'PROCESSED',
            data
          };
        }), (0, _rxjs.catchError)(error => {
          return (0, _rxjs.of)({
            status: 'ERROR',
            error
          });
        }));
      }))).pipe((0, _rxjs.takeUntil)(this.destroy$), (0, _rxjs.tap)(processingStatus => {
        resource.onProcessingStatus && resource.onProcessingStatus(processingStatus, {});
        this.setState(prevState => {
          const newState = (0, _immer.default)(draft => {
            draft.resources[resource.key].status = processingStatus.status;
            draft.resources[resource.key].data = processingStatus.data;
            draft.resources[resource.key].error = processingStatus.error;
          })(prevState);
          this.newState = newState;
          return newState;
        });
      })).subscribe();
    }), 1);
    /******************************************************************************************************* */

    const adaptLoadData = loadConfig.entity.adaptData ? loadConfig.entity.adaptData : data => ({
      initialData: data,
      resources: {}
    });
    const processingStatus$ = new _rxjs.Subject();
    const params$ = this.props.params$ ? (0, _rxjs.merge)(this.props.params$, this.triggerLoad$) : this.triggerLoad$;
    (0, _rxjs.merge)(processingStatus$.pipe((0, _rxjs.mapTo)({
      status: 'PROCESSING'
    })), params$.pipe((0, _rxjs.takeUntil)(this.destroy$), (0, _rxjs.tap)(params => {
      this.setState(prevState => {
        const newState = (0, _immer.default)(draft => {
          draft.params = params;
        })(prevState);
        this.newState = newState;
        return newState;
      });
      processingStatus$.next();
    }), (0, _rxjs.switchMap)(params => {
      const loadConfig = this.props.load ? this.props.load : {
        entity: {}
      };
      const retryConfig = loadConfig.entity.retry ? loadConfig.entity.retry : {
        interval: 200
      };
      const data$ = loadConfig.entity.data$ ? loadConfig.entity.data$(params) : (0, _rxjs.of)(params);
      const loadData$ = _lang.ModuleUtils.isObservable(data$) ? data$ : _lang.ModuleUtils.isPromise(data$) ? (0, _rxjs.from)(data$) : (0, _rxjs.of)(data$);
      return loadData$.pipe(retryProcessing(retryConfig), (0, _rxjs.take)(1), (0, _rxjs.map)((data, e) => {
        return adaptLoadData(data);
      }), (0, _rxjs.map)(data => {
        return {
          status: 'PROCESSED',
          data
        };
      }), (0, _rxjs.catchError)(error => {
        return (0, _rxjs.of)({
          status: 'ERROR',
          error
        });
      }));
    }))).pipe((0, _rxjs.takeUntil)(this.destroy$), (0, _rxjs.tap)(processingStatus => {
      loadConfig.entity.onProcessingStatus && loadConfig.entity.onProcessingStatus(processingStatus, this.newState.params);

      if (processingStatus.status === 'PROCESSED') {
        this.setState(prevState => {
          const newState = (0, _immer.default)(draft => {
            const resources = processingStatus.data.resources;
            Object.keys(resources).map(key => ({
              key,
              data: resources[key]
            })).forEach(item => {
              draft.resources[item.key] = {
                status: 'PROCESSED',
                data: item.data
              };
            });
          })(prevState);
          this.newState = newState;
          return newState;
        });
      }

      this.setState(prevState => {
        const newState = (0, _immer.default)(draft => {
          var _processingStatus$dat;

          draft.loadStatus.status = processingStatus.status;
          draft.loadStatus.data = (_processingStatus$dat = processingStatus.data) == null ? void 0 : _processingStatus$dat.initialData;
          draft.loadStatus.error = processingStatus.error;
        })(prevState);
        this.newState = newState;
        return newState;
      });
    })).subscribe();
    /**************************************************************** */
  }

  configLoad() {
    const processingStatus$ = new _rxjs.Subject();
    this.triggerLoad$.pipe();
  }

  configSave() {
    const saveConfig = this.props.save;
    const processingStatus$ = new _rxjs.Subject();
    (0, _rxjs.merge)(processingStatus$.pipe((0, _rxjs.mapTo)({
      status: 'PROCESSING'
    })), this.triggerSave$.pipe((0, _rxjs.tap)(x => {
      processingStatus$.next();
    }), (0, _rxjs.exhaustMap)(data => {
      const data$ = saveConfig.saveDataFn$(data, this.newState.params);
      const saveData$ = _lang.ModuleUtils.isObservable(data$) ? data$ : _lang.ModuleUtils.isPromise(data$) ? (0, _rxjs.from)(data$) : (0, _rxjs.of)(data$);
      return saveData$.pipe((0, _rxjs.map)(resp => ({
        status: 'PROCESSED',
        data: resp
      })), (0, _rxjs.catchError)(error => (0, _rxjs.of)({
        status: 'ERROR',
        error
      })));
    }))).pipe((0, _rxjs.tap)(processingStatus => {
      saveConfig && saveConfig.onProcessingStatus && saveConfig.onProcessingStatus(processingStatus, this.newState.params);
      this.setState(prevState => {
        const newState = (0, _immer.default)(draft => {
          draft.saveStatus.status = processingStatus.status;
          draft.saveStatus.data = processingStatus.data;
          draft.saveStatus.error = processingStatus.error;
        })(prevState);
        this.newState = newState;
        return newState;
      });
    })).subscribe();
  }

  close() {
    this.destroy$.next();
    this.destroy$.complete();
  }

  submit() {
    this.setState(prevState => {
      const newState = (0, _immer.default)(draft => {
        draft.submitted = true;
      })(prevState);
      this.newState = newState;
      return newState;
    });
  }

  resetSubmit() {
    this.setState(prevState => {
      const newState = (0, _immer.default)(draft => {
        draft.submitted = false;
      })(prevState);
      this.newState = newState;
      return newState;
    });
  }

  resetForm() {// this.setState(
    //     (prevState) => {
    //         const newState = produce(
    //             (draft: IEntityFormState) => {
    //                 draft.data = this.newState.loadStatus.data;
    //             }
    //         )(prevState)
    //         this.newState = newState;
    //         return newState;
    //     }
    // )
  }

  get formState() {
    return this.newState;
  }

  get resources() {
    return this.newState.resources;
  }

  reloadResource(resourceId) {
    this.reloadResource$.next({
      id: resourceId,
      params: this.newState.resourceParams[resourceId]
    });
  }

  get config() {
    var _this$formState;

    return {
      formState: this.formState,
      resources: (_this$formState = this.formState) == null ? void 0 : _this$formState.resources,
      reloadResource: this.reloadResource,
      submit: this.submit.bind(this),
      resetSubmit: this.resetSubmit,
      resetForm: this.resetForm,
      triggerSave: this.triggerSave$.next.bind(this.triggerSave$),
      triggerLoad: this.triggerLoad$.next.bind(this.triggerLoad$),
      destroy$: this.destroy$,
      loadResource: this.loadResource$.next.bind(this.loadResource$)
    };
  }

}

exports.EntityFormClass = EntityFormClass;